@@ -86,6 +86,14 @@ extern void _xfdashboard_marshal_VOID__INT_INT (GClosure     *closure,
                                                 gpointer      invocation_hint,
                                                 gpointer      marshal_data);
 
+/* VOID:STRING,BOOLEAN (marshal.list:11) */
+extern void _xfdashboard_marshal_VOID__STRING_BOOLEAN (GClosure     *closure,
+                                                       GValue       *return_value,
+                                                       guint         n_param_values,
+                                                       const GValue *param_values,
+                                                       gpointer      invocation_hint,
+                                                       gpointer      marshal_data);
+
 G_END_DECLS
 
 #endif /* ___xfdashboard_marshal_MARSHAL_H__ */
