@@ -2,7 +2,7 @@
  * search-manager: Single-instance managing search providers and
  *                 handles search requests
  * 
- * Copyright 2012-2015 Stephan Haller <nomad@froevel.de>
+ * Copyright 2012-2016 Stephan Haller <nomad@froevel.de>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -389,6 +389,11 @@ GObject* xfdashboard_search_manager_create_provider(XfdashboardSearchManager *se
 
 	/* Create search provider */
 	provider=g_object_new(data->gtype, "provider-id", data->ID, NULL);
+	if(provider &&
+		XFDASHBOARD_SEARCH_PROVIDER_GET_CLASS(provider)->initialize)
+	{
+		XFDASHBOARD_SEARCH_PROVIDER_GET_CLASS(provider)->initialize(XFDASHBOARD_SEARCH_PROVIDER(provider));
+	}
 
 	/* Return newly created search provider */
 	return(provider);
