@@ -1,7 +1,7 @@
 /*
  * search-result-set: Contains and manages set of identifiers of a search
  * 
- * Copyright 2012-2015 Stephan Haller <nomad@froevel.de>
+ * Copyright 2012-2016 Stephan Haller <nomad@froevel.de>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,6 +49,76 @@ struct _XfdashboardSearchResultSetPrivate
 };
 
 /* IMPLEMENTATION: Private variables and methods */
+typedef struct _XfdashboardSearchResultSetItemData		XfdashboardSearchResultSetItemData;
+struct _XfdashboardSearchResultSetItemData
+{
+	gint									refCount;
+
+	/* Item related */
+	gfloat									score;
+};
+
+/* Create, destroy, ref and unref item data for an item */
+static XfdashboardSearchResultSetItemData* _xfdashboard_search_result_set_item_data_new(void)
+{
+	XfdashboardSearchResultSetItemData	*data;
+
+	/* Create statistics data */
+	data=g_new0(XfdashboardSearchResultSetItemData, 1);
+	if(!data) return(NULL);
+
+	/* Set up statistics data */
+	data->refCount=1;
+
+	return(data);
+}
+
+static void _xfdashboard_search_result_set_item_data_free(XfdashboardSearchResultSetItemData *inData)
+{
+	g_return_if_fail(inData);
+
+	/* Release common allocated resources */
+	g_free(inData);
+}
+
+static XfdashboardSearchResultSetItemData* _xfdashboard_search_result_set_item_data_ref(XfdashboardSearchResultSetItemData *inData)
+{
+	g_return_val_if_fail(inData, NULL);
+
+	inData->refCount++;
+	return(inData);
+}
+
+static void _xfdashboard_search_result_set_item_data_unref(XfdashboardSearchResultSetItemData *inData)
+{
+	g_return_if_fail(inData);
+
+	inData->refCount--;
+	if(inData->refCount==0) _xfdashboard_search_result_set_item_data_free(inData);
+}
+
+static XfdashboardSearchResultSetItemData* _xfdashboard_search_result_set_item_data_get(XfdashboardSearchResultSet *self, GVariant *inItem)
+{
+	XfdashboardSearchResultSetPrivate		*priv;
+	XfdashboardSearchResultSetItemData		*itemData;
+
+	g_return_val_if_fail(XFDASHBOARD_IS_SEARCH_RESULT_SET(self), NULL);
+	g_return_val_if_fail(inItem, NULL);
+
+	priv=self->priv;
+	itemData=NULL;
+
+	/* Lookup item in result and take reference on item data if found
+	 * before returning it,
+	 */
+	if(g_hash_table_lookup_extended(priv->set, inItem, NULL, (gpointer*)&itemData))
+	{
+		_xfdashboard_search_result_set_item_data_ref(itemData);
+	}
+
+	/* Return item data found for item in result set if any */
+	return(itemData);
+}
 
 /* Internal callback function for calling callback functions for sorting */
 static gint _xfdashboard_search_result_set_sort_internal(gconstpointer inLeft,
@@ -58,13 +128,37 @@ static gint _xfdashboard_search_result_set_sort_internal(gconstpointer inLeft,
 	XfdashboardSearchResultSet				*self=XFDASHBOARD_SEARCH_RESULT_SET(inUserData);
 	XfdashboardSearchResultSetPrivate		*priv=self->priv;
 	GVariant								*left;
+	XfdashboardSearchResultSetItemData		*leftData;
 	GVariant								*right;
+	XfdashboardSearchResultSetItemData		*rightData;
+	gint									result;
+
+	result=0;
 
 	/* Get items to compare */
 	left=(GVariant*)inLeft;
 	right=(GVariant*)inRight;
 
-	/* Call sorting callback function now */
+	/* Get score for each item to compare for sorting if for both items available */
+	leftData=_xfdashboard_search_result_set_item_data_get(self, left);
+	rightData=_xfdashboard_search_result_set_item_data_get(self, right);
+	if(leftData && rightData)
+	{
+		/* Set result to corresponding value and other than null if the
+		 * scores are not equal.
+		 */
+		if(leftData->score < rightData->score) result=1;
+		if(leftData->score > rightData->score) result=-1;
+	}
+	if(leftData) _xfdashboard_search_result_set_item_data_unref(leftData);
+	if(rightData) _xfdashboard_search_result_set_item_data_unref(rightData);
+
+	/* If both items do not have the same score the result is set to value
+	 * other than zero. So return it now.
+	 */
+	if(result!=0) return(result);
+
+	/* Call sorting callback function now if both have the same score */
 	return((priv->sortCallback)(left, right, priv->sortUserData));
 }
 
@@ -132,7 +226,7 @@ static void xfdashboard_search_result_set_init(XfdashboardSearchResultSet *self)
 	priv->set=g_hash_table_new_full(g_variant_hash,
 									g_variant_equal,
 									(GDestroyNotify)g_variant_unref,
-									NULL);
+									(GDestroyNotify)_xfdashboard_search_result_set_item_data_unref);
 }
 
 /* IMPLEMENTATION: Public API */
@@ -155,6 +249,7 @@ guint xfdashboard_search_result_set_get_size(XfdashboardSearchResultSet *self)
 void xfdashboard_search_result_set_add_item(XfdashboardSearchResultSet *self, GVariant *inItem)
 {
 	XfdashboardSearchResultSetPrivate		*priv;
+	XfdashboardSearchResultSetItemData		*itemData;
 
 	g_return_if_fail(XFDASHBOARD_IS_SEARCH_RESULT_SET(self));
 	g_return_if_fail(inItem);
@@ -162,7 +257,14 @@ void xfdashboard_search_result_set_add_item(XfdashboardSearchResultSet *self, GV
 	priv=self->priv;
 
 	/* Add or replace item in hash table */
-	g_hash_table_insert(priv->set, g_variant_ref_sink(inItem), GINT_TO_POINTER(1));
+	if(!g_hash_table_lookup_extended(priv->set, inItem, NULL, (gpointer*)&itemData))
+	{
+		/* Create data for item to add */
+		itemData=_xfdashboard_search_result_set_item_data_new();
+
+		/* Add new item to result set */
+		g_hash_table_insert(priv->set, g_variant_ref_sink(inItem), itemData);
+	}
 }
 
 /* Check if a result item exists already in result set */
@@ -212,6 +314,7 @@ GList* xfdashboard_search_result_set_get_all(XfdashboardSearchResultSet *self)
 	/* Return result */
 	return(list);
 }
+
 /* Get list of all items existing in both result sets.
  * Returned list should be freed with g_list_free_full(result, g_variant_unref)
  */
@@ -338,3 +441,57 @@ void xfdashboard_search_result_set_set_sort_func_full(XfdashboardSearchResultSet
 		priv->sortUserDataDestroyFunc=inUserDataDestroyFunc;
 	}
 }
+
+/* Get/set score for a result item in result set */
+gfloat xfdashboard_search_result_set_get_item_score(XfdashboardSearchResultSet *self, GVariant *inItem)
+{
+	gfloat									score;
+	XfdashboardSearchResultSetItemData		*itemData;
+
+	g_return_val_if_fail(XFDASHBOARD_IS_SEARCH_RESULT_SET(self), 0.0f);
+	g_return_val_if_fail(inItem, 0.0f);
+
+	score=0.0f;
+
+	/* Check if requested item exists and get its score from item data */
+	itemData=_xfdashboard_search_result_set_item_data_get(self, inItem);
+	if(itemData)
+	{
+		score=itemData->score;
+
+		/* Release allocated resources */
+		_xfdashboard_search_result_set_item_data_unref(itemData);
+	}
+
+	/* Return score of item */
+	return(score);
+}
+
+gboolean xfdashboard_search_result_set_set_item_score(XfdashboardSearchResultSet *self, GVariant *inItem, gfloat inScore)
+{
+	XfdashboardSearchResultSetItemData		*itemData;
+	gboolean								success;
+
+	g_return_val_if_fail(XFDASHBOARD_IS_SEARCH_RESULT_SET(self), FALSE);
+	g_return_val_if_fail(inItem, FALSE);
+	g_return_val_if_fail(inScore>=0.0f && inScore<=1.0f, FALSE);
+
+	success=FALSE;
+
+	/* Check if requested item exists and set score at its item data */
+	itemData=_xfdashboard_search_result_set_item_data_get(self, inItem);
+	if(itemData)
+	{
+		/* Set score */
+		itemData->score=inScore;
+
+		/* Release allocated resources */
+		_xfdashboard_search_result_set_item_data_unref(itemData);
+
+		/* Set flag that item exists in result set and data could be set */
+		success=TRUE;
+	}
+
+	/* Return success state for item */
+	return(success);
+}
