@@ -1,7 +1,7 @@
 /*
  * search-view: A view showing applications matching search criterias
  * 
- * Copyright 2012-2015 Stephan Haller <nomad@froevel.de>
+ * Copyright 2012-2016 Stephan Haller <nomad@froevel.de>
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,8 +34,6 @@
 #include "focusable.h"
 #include "utils.h"
 #include "search-result-container.h"
-#include "click-action.h"
-#include "drag-action.h"
 #include "focus-manager.h"
 #include "enums.h"
 #include "application.h"
@@ -101,7 +99,6 @@ struct _XfdashboardSearchViewProviderData
 	XfdashboardSearchResultSet			*lastResultSet;
 
 	ClutterActor						*container;
-	GHashTable							*mapping;
 };
 
 struct _XfdashboardSearchViewSearchTerms
@@ -137,40 +134,6 @@ static gboolean _xfdashboard_search_view_on_repaint_after_update_callback(gpoint
 	return(G_SOURCE_REMOVE);
 }
 
-/* Helper to clean up a search provider's container by disconnecting all signals to
- * its result items' actors and clean mapping hash table between actors and
- * result items.
- */
-static void _xfdashboard_search_view_clean_provider_container(XfdashboardSearchViewProviderData *inProviderData)
-{
-	g_return_if_fail(inProviderData);
-
-	/* Iterate through mapping, if available, and for each key disconnect signal
-	 * from the value (which is a ClutterActor) and take a also an additional
-	 * reference at actor because removing it from mapping hash table will unreference
-	 * it again.
-	 */
-	if(inProviderData->mapping)
-	{
-		GHashTableIter		hashIter;
-		GVariant			*key;
-		ClutterActor		*value;
-
-		g_hash_table_iter_init(&hashIter, inProviderData->mapping);
-		while(g_hash_table_iter_next(&hashIter, (gpointer*)&key, (gpointer*)&value))
-		{
-			/* First disconnect signal handlers from actor before modifying mapping hash table */
-			g_signal_handlers_disconnect_by_data(value, inProviderData);
-
-			/* Take a reference on value (it is the destroyed actor) because removing
-			 * key from mapping causes unrefencing key and value.
-			 */
-			g_object_ref(value);
-			g_hash_table_iter_remove(&hashIter);
-		}
-	}
-}
-
 /* Create search term data for a search string */
 static XfdashboardSearchViewSearchTerms* _xfdashboard_search_view_search_terms_new(const gchar *inSearchString)
 {
@@ -247,10 +210,6 @@ static XfdashboardSearchViewProviderData* _xfdashboard_search_view_provider_data
 	data->lastTerms=NULL;
 	data->lastResultSet=NULL;
 	data->container=NULL;
-	data->mapping=g_hash_table_new_full(g_variant_hash,
-										g_variant_equal,
-										(GDestroyNotify)g_variant_unref,
-										(GDestroyNotify)g_object_unref);
 
 	return(data);
 }
@@ -272,9 +231,6 @@ static void _xfdashboard_search_view_provider_data_free(XfdashboardSearchViewPro
 	}
 #endif
 
-	/* Clean provider's container and move focus if necessary */
-	_xfdashboard_search_view_clean_provider_container(inData);
-
 	/* Destroy container */
 	if(inData->container)
 	{
@@ -287,7 +243,6 @@ static void _xfdashboard_search_view_provider_data_free(XfdashboardSearchViewPro
 	}
 
 	/* Release allocated resources */
-	if(inData->mapping) g_hash_table_unref(inData->mapping);
 	if(inData->lastResultSet) g_object_unref(inData->lastResultSet);
 	if(inData->lastTerms) _xfdashboard_search_view_search_terms_unref(inData->lastTerms);
 	if(inData->provider) g_object_unref(inData->provider);
@@ -332,7 +287,7 @@ static XfdashboardSearchViewProviderData* _xfdashboard_search_view_get_provider_
 		data=(XfdashboardSearchViewProviderData*)iter->data;
 
 		if(data->provider &&
-			xfdashboard_search_provider_has_id(data->provider, inProviderID)==0)
+			xfdashboard_search_provider_has_id(data->provider, inProviderID))
 		{
 			return(_xfdashboard_search_view_provider_data_ref(data));
 		}
@@ -456,49 +411,41 @@ static void _xfdashboard_search_view_on_search_provider_unregistered(Xfdashboard
 }
 
 /* A result item actor was clicked */
-static void _xfdashboard_search_view_on_provider_item_actor_clicked(XfdashboardClickAction *inAction,
-																	ClutterActor *inActor,
-																	gpointer inUserData)
+static void _xfdashboard_search_view_on_result_item_clicked(XfdashboardSearchResultContainer *inContainer,
+															GVariant *inItem,
+															ClutterActor *inActor,
+															gpointer inUserData)
 {
 	XfdashboardSearchView				*self;
 	XfdashboardSearchViewPrivate		*priv;
 	XfdashboardSearchViewProviderData	*providerData;
-	GHashTableIter						iter;
-	GVariant							*key;
-	ClutterActor						*value;
 	const gchar							**searchTerms;
+	gboolean							success;
 
+	g_return_if_fail(XFDASHBOARD_IS_SEARCH_RESULT_CONTAINER(inContainer));
+	g_return_if_fail(inItem);
 	g_return_if_fail(CLUTTER_IS_ACTOR(inActor));
 	g_return_if_fail(inUserData);
 
 	providerData=(XfdashboardSearchViewProviderData*)inUserData;
 
-	/* Iterate through mapping and at first key whose value matching actor
-	 * tell provider to activate item.
-	 */
-	g_hash_table_iter_init(&iter, providerData->mapping);
-	while(g_hash_table_iter_next(&iter, (gpointer*)&key, (gpointer*)&value))
+	/* Get search view and private data of view */
+	self=providerData->view;
+	priv=self->priv;
+
+	/* Get search terms to pass them to search provider */
+	searchTerms=NULL;
+	if(priv->lastTerms) searchTerms=(const gchar**)priv->lastTerms->termList;
+
+	/* Tell provider to launch search */
+	success=xfdashboard_search_provider_activate_result(providerData->provider,
+														inItem,
+														inActor,
+														searchTerms);
+	if(success)
 	{
-		/* If value of key iterated matches actor activate item */
-		if(value==inActor)
-		{
-			/* Get search view and private data of view */
-			self=providerData->view;
-			priv=self->priv;
-
-			/* Get search terms to pass them to search provider */
-			searchTerms=NULL;
-			if(priv->lastTerms) searchTerms=(const gchar**)priv->lastTerms->termList;
-
-			/* Tell provider that a result item was clicked */
-			xfdashboard_search_provider_activate_result(providerData->provider,
-															key,
-															inActor,
-															searchTerms);
-
-			/* All done so return here */
-			return;
-		}
+		/* Activating result item seems to be successfuly so quit application */
+		xfdashboard_application_quit();
 	}
 }
 
@@ -510,6 +457,7 @@ static void _xfdashboard_search_view_on_provider_icon_clicked(XfdashboardSearchR
 	XfdashboardSearchViewPrivate		*priv;
 	XfdashboardSearchViewProviderData	*providerData;
 	const gchar							**searchTerms;
+	gboolean							success;
 
 	g_return_if_fail(XFDASHBOARD_IS_SEARCH_RESULT_CONTAINER(inContainer));
 	g_return_if_fail(inUserData);
@@ -525,7 +473,12 @@ static void _xfdashboard_search_view_on_provider_icon_clicked(XfdashboardSearchR
 	if(priv->lastTerms) searchTerms=(const gchar**)priv->lastTerms->termList;
 
 	/* Tell provider to launch search */
-	xfdashboard_search_provider_launch_search(providerData->provider, searchTerms);
+	success=xfdashboard_search_provider_launch_search(providerData->provider, searchTerms);
+	if(success)
+	{
+		/* Activating result item seems to be successfuly so quit application */
+		xfdashboard_application_quit();
+	}
 }
 
 /* A container of a provider is going to be destroyed */
@@ -547,7 +500,7 @@ static void _xfdashboard_search_view_on_provider_container_destroyed(ClutterActo
 	/* Move selection to first selectable actor at next available container
 	 * if this provider whose container to destroy is the currently selected one.
 	 * This avoids reselecting the next available actor in container when
-	 * we container's children will get destroyed (one of the actors is the
+	 * the container's children will get destroyed (one of the actors is the
 	 * current selection and from then on reselection will happen.)
 	 */
 	if(priv->selectionProvider==providerData)
@@ -594,7 +547,8 @@ static void _xfdashboard_search_view_on_provider_container_destroyed(ClutterActo
 				selectableActor=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(iterProviderData->container),
 																					NULL,
 																					XFDASHBOARD_SELECTION_TARGET_FIRST,
-																					XFDASHBOARD_VIEW(self));
+																					XFDASHBOARD_VIEW(self),
+																					FALSE);
 				if(selectableActor)
 				{
 					newSelection=selectableActor;
@@ -619,7 +573,8 @@ static void _xfdashboard_search_view_on_provider_container_destroyed(ClutterActo
 				selectableActor=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(iterProviderData->container),
 																					NULL,
 																					XFDASHBOARD_SELECTION_TARGET_FIRST,
-																					XFDASHBOARD_VIEW(self));
+																					XFDASHBOARD_VIEW(self),
+																					FALSE);
 				if(selectableActor)
 				{
 					newSelection=selectableActor;
@@ -643,130 +598,13 @@ static void _xfdashboard_search_view_on_provider_container_destroyed(ClutterActo
 		xfdashboard_focusable_set_selection(XFDASHBOARD_FOCUSABLE(self), newSelection);
 	}
 
-	/* Clean provider's container and move focus if necessary */
-	_xfdashboard_search_view_clean_provider_container(providerData);
-
 	/* Container will be destroyed so unset pointer to it at provider */
 	providerData->container=NULL;
 }
 
-/* A result item actor is going to be destroyed */
-static void _xfdashboard_search_view_on_provider_item_actor_destroy(ClutterActor *inActor, gpointer inUserData)
-{
-	XfdashboardSearchViewProviderData	*providerData;
-	GHashTableIter						iter;
-	GVariant							*key;
-	ClutterActor						*value;
-
-	g_return_if_fail(CLUTTER_IS_ACTOR(inActor));
-	g_return_if_fail(inUserData);
-
-	providerData=(XfdashboardSearchViewProviderData*)inUserData;
-
-	/* First disconnect signal handlers from actor before modifying mapping hash table */
-	g_signal_handlers_disconnect_by_data(inActor, providerData);
-
-	/* Iterate through mapping and remove each key whose value matches actor
-	 * going to be destroyed and remove it from mapping hash table.
-	 */
-	g_hash_table_iter_init(&iter, providerData->mapping);
-	while(g_hash_table_iter_next(&iter, (gpointer*)&key, (gpointer*)&value))
-	{
-		/* If value of key iterated matches destroyed actor remove it from mapping */
-		if(value==inActor)
-		{
-			/* Take a reference on value (it is the destroyed actor) because removing
-			 * key from mapping causes unrefencing key and value.
-			 */
-			g_object_ref(inActor);
-			g_hash_table_iter_remove(&iter);
-		}
-	}
-}
-
 /* Updates container of provider with new result set from a last search.
  * Also creates or destroys the container for search provider if needed.
  */
-static ClutterActor* _xfdashboard_search_view_update_provider_actor_new(XfdashboardSearchView *self,
-																			XfdashboardSearchViewProviderData *inProviderData,
-																			GVariant *inItem)
-{
-	ClutterActor		*actor;
-	ClutterAction		*action;
-	GList				*actions;
-	GList				*actionsIter;
-
-	g_return_val_if_fail(XFDASHBOARD_IS_SEARCH_VIEW(self), NULL);
-	g_return_val_if_fail(inProviderData, NULL);
-	g_return_val_if_fail(inItem, NULL);
-
-	/* Create actor for item */
-	actor=xfdashboard_search_provider_create_result_actor(inProviderData->provider, inItem);
-	if(!actor)
-	{
-		gchar			*itemText;
-
-		itemText=g_variant_print(inItem, TRUE);
-		g_warning(_("Failed to add actor for result item %s of provider %s: Could not create actor"),
-					itemText,
-					G_OBJECT_TYPE_NAME(inProviderData->provider));
-		g_free(itemText);
-
-		return(NULL);
-	}
-
-	if(!CLUTTER_IS_ACTOR(actor))
-	{
-		gchar			*itemText;
-
-		itemText=g_variant_print(inItem, TRUE);
-		g_critical(_("Failed to add actor for result item %s of provider %s: Actor of type %s is not derived from class %s"),
-					itemText,
-					G_OBJECT_TYPE_NAME(inProviderData->provider),
-					G_IS_OBJECT(actor) ? G_OBJECT_TYPE_NAME(actor) : "<unknown>",
-					g_type_name(CLUTTER_TYPE_ACTOR));
-		g_free(itemText);
-
-		/* Release allocated resources */
-		clutter_actor_destroy(actor);
-
-		return(NULL);
-	}
-
-	/* Connect to 'destroy' signal of actor to remove it from mapping hash table
-	 * if actor was destroyed (accidently)
-	 */
-	g_signal_connect(actor,
-						"destroy",
-						G_CALLBACK(_xfdashboard_search_view_on_provider_item_actor_destroy),
-						inProviderData);
-
-	/* Add click action to actor and connect signal */
-	action=xfdashboard_click_action_new();
-	clutter_actor_add_action(actor, action);
-	g_signal_connect(action,
-						"clicked",
-						G_CALLBACK(_xfdashboard_search_view_on_provider_item_actor_clicked),
-						inProviderData);
-
-	/* Iterate through all actions of actor and if it has an action of type
-	 * XfdashboardDragAction and has no source set then set this view as source
-	 */
-	actions=clutter_actor_get_actions(actor);
-	for(actionsIter=actions; actionsIter; actionsIter=g_list_next(actionsIter))
-	{
-		if(XFDASHBOARD_IS_DRAG_ACTION(actionsIter->data) &&
-			!xfdashboard_drag_action_get_source(XFDASHBOARD_DRAG_ACTION(actionsIter->data)))
-		{
-			g_object_set(actionsIter->data, "source", self, NULL);
-		}
-	}
-	if(actions) g_list_free(actions);
-
-	/* Return newly created and set up actor */
-	return(actor);
-}
-
 static void _xfdashboard_search_view_update_provider_container(XfdashboardSearchView *self,
 																XfdashboardSearchViewProviderData *inProviderData,
 																XfdashboardSearchResultSet *inNewResultSet)
@@ -781,119 +619,38 @@ static void _xfdashboard_search_view_update_provider_container(XfdashboardSearch
 	if(inNewResultSet &&
 		xfdashboard_search_result_set_get_size(inNewResultSet)>0)
 	{
-		GList					*allList;
-		GList					*removeList;
-		GList					*iter;
-		ClutterActor			*actor;
-		ClutterActor			*lastActor;
-
-		/* Create container for provider if it does not exist yet */
+		/* Create container for search provider if it does not exist yet */
 		if(!inProviderData->container)
 		{
-			actor=xfdashboard_search_result_container_new(inProviderData->provider);
-			if(!actor) return;
+			/* Create container for search provider */
+			inProviderData->container=xfdashboard_search_result_container_new(inProviderData->provider);
+			if(!inProviderData->container) return;
 
-			inProviderData->container=actor;
+			/* Add new container to search view */
 			clutter_actor_add_child(CLUTTER_ACTOR(self), inProviderData->container);
 
+			/* Connect signals */
 			g_signal_connect(inProviderData->container,
 								"icon-clicked",
 								G_CALLBACK(_xfdashboard_search_view_on_provider_icon_clicked),
 								inProviderData);
 
+			g_signal_connect(inProviderData->container,
+								"item-clicked",
+								G_CALLBACK(_xfdashboard_search_view_on_result_item_clicked),
+								inProviderData);
+
 			g_signal_connect(inProviderData->container,
 								"destroy",
 								G_CALLBACK(_xfdashboard_search_view_on_provider_container_destroyed),
 								inProviderData);
 		}
 
-		/* Get list of all items in result set for and a list of items whose actors
-		 * to remove from container.
-		 */
-		allList=xfdashboard_search_result_set_get_all(inNewResultSet);
-
-		removeList=NULL;
-		if(inProviderData->lastResultSet)
-		{
-			removeList=xfdashboard_search_result_set_complement(inNewResultSet, inProviderData->lastResultSet);
-		}
-
-		/* Create actor for each item in list which is new to mapping */
-		lastActor=NULL;
-		for(iter=allList; iter; iter=g_list_next(iter))
-		{
-			GVariant			*item;
-
-			/* Get item to add */
-			item=(GVariant*)iter->data;
-
-			/* If item does not exist in mapping then create actor and add it to mapping */
-			if(!g_hash_table_lookup_extended(inProviderData->mapping, item, NULL, (gpointer*)&actor))
-			{
-				actor=_xfdashboard_search_view_update_provider_actor_new(self, inProviderData, item);
-				if(actor)
-				{
-					/* Add newly created actor to container of provider */
-					xfdashboard_search_result_container_add_result_actor(XFDASHBOARD_SEARCH_RESULT_CONTAINER(inProviderData->container),
-																			actor,
-																			lastActor);
-					g_hash_table_insert(inProviderData->mapping, g_variant_ref(item), g_object_ref(actor));
-				}
-			}
-
-			/* Remember either existing actor from hash table lookup or
-			 * the newly created actor as the last one seen.
-			 */
-			if(actor) lastActor=actor;
-		}
-
-		/* Remove actors for each item in list of removing items */
-		for(iter=removeList; iter; iter=g_list_next(iter))
-		{
-			GVariant			*item;
-
-			/* Get item to add */
-			item=(GVariant*)iter->data;
-
-			/* Get actor to remove */
-			if(g_hash_table_lookup_extended(inProviderData->mapping, item, NULL, (gpointer*)&actor))
-			{
-				if(!CLUTTER_IS_ACTOR(actor))
-				{
-					gchar		*itemText;
-
-					itemText=g_variant_print(item, TRUE);
-					g_critical(_("Failed to remove actor for result item %s of provider %s: Actor of type %s is not derived from class %s"),
-								itemText,
-								G_OBJECT_TYPE_NAME(inProviderData->provider),
-								G_IS_OBJECT(actor) ? G_OBJECT_TYPE_NAME(actor) : "<unknown>",
-								g_type_name(CLUTTER_TYPE_ACTOR));
-					g_free(itemText);
-
-					continue;
-				}
-
-				/* First disconnect signal handlers from actor before modifying mapping hash table */
-				g_signal_handlers_disconnect_by_data(actor, inProviderData);
-
-				/* Remove actor from mapping hash table before destroying it */
-				g_hash_table_remove(inProviderData->mapping, item);
-
-				/* Destroy actor and remove from hash table */
-				clutter_actor_destroy(actor);
-			}
-		}
-
-		/* Release allocated resources */
-		if(removeList) g_list_free_full(removeList, (GDestroyNotify)g_variant_unref);
-		if(allList) g_list_free_full(allList, (GDestroyNotify)g_variant_unref);
+		xfdashboard_search_result_container_update(XFDASHBOARD_SEARCH_RESULT_CONTAINER(inProviderData->container), inNewResultSet);
 	}
 		/* ... but if no result set for provider is given then destroy existing container */
 		else
 		{
-			/* Clean provider's container and move focus if necessary */
-			_xfdashboard_search_view_clean_provider_container(inProviderData);
-
 			/* Destroy container */
 			if(inProviderData->container)
 			{
@@ -1002,7 +759,7 @@ static guint _xfdashboard_search_view_perform_search(XfdashboardSearchView *self
 			_xfdashboard_search_view_can_do_incremental_search(providerData->lastTerms, inSearchTerms))
 		{
 			canDoIncrementalSearch=TRUE;
-			providerLastResultSet=g_object_ref(providerData->lastResultSet);
+			if(providerData->lastResultSet) providerLastResultSet=g_object_ref(providerData->lastResultSet);
 		}
 
 		/* Perform search */
@@ -1012,18 +769,18 @@ static guint _xfdashboard_search_view_perform_search(XfdashboardSearchView *self
 		g_debug("Performed %s search at search provider %s and got %u result items",
 					canDoIncrementalSearch==TRUE ? "incremental" : "full",
 					G_OBJECT_TYPE_NAME(providerData->provider),
-					xfdashboard_search_result_set_get_size(providerNewResultSet));
+					providerNewResultSet ? xfdashboard_search_result_set_get_size(providerNewResultSet) : 0);
 
 		/* Count number of results */
-		numberResults+=xfdashboard_search_result_set_get_size(providerNewResultSet);
-
-		/* Update view of search provider for new result set */
-		_xfdashboard_search_view_update_provider_container(self, providerData, providerNewResultSet);
+		if(providerNewResultSet) numberResults+=xfdashboard_search_result_set_get_size(providerNewResultSet);
 
 		/* Remember new search term as last one at search provider */
 		if(providerData->lastTerms) _xfdashboard_search_view_search_terms_unref(providerData->lastTerms);
 		providerData->lastTerms=_xfdashboard_search_view_search_terms_ref(inSearchTerms);
 
+		/* Update view of search provider for new result set */
+		_xfdashboard_search_view_update_provider_container(self, providerData, providerNewResultSet);
+
 		/* Release allocated resources */
 		if(providerLastResultSet) g_object_unref(providerLastResultSet);
 		if(providerNewResultSet) g_object_unref(providerNewResultSet);
@@ -1143,8 +900,6 @@ static gboolean _xfdashboard_search_view_focusable_can_focus(XfdashboardFocusabl
 	/* If this view is not enabled it is not focusable */
 	if(!xfdashboard_view_get_enabled(XFDASHBOARD_VIEW(self))) return(FALSE);
 
-	// TODO: We can only focus this view if at least one provider has an existing container
-
 	/* If we get here this actor can be focused */
 	return(TRUE);
 }
@@ -1267,6 +1022,180 @@ static gboolean _xfdashboard_search_view_focusable_set_selection(XfdashboardFocu
 }
 
 /* Find requested selection target depending of current selection */
+static ClutterActor* _xfdashboard_search_view_focusable_find_selection_internal_backwards(XfdashboardSearchView *self,
+																							XfdashboardSearchResultContainer *inContainer,
+																							ClutterActor *inSelection,
+																							XfdashboardSelectionTarget inDirection,
+																							GList *inCurrentProviderIter,
+																							XfdashboardSelectionTarget inNextContainerDirection)
+{
+	ClutterActor							*newSelection;
+	GList									*iter;
+	XfdashboardSearchViewProviderData		*providerData;
+
+	g_return_val_if_fail(XFDASHBOARD_IS_SEARCH_VIEW(self), NULL);
+	g_return_val_if_fail(XFDASHBOARD_IS_SEARCH_RESULT_CONTAINER(inContainer), NULL);
+	g_return_val_if_fail(CLUTTER_IS_ACTOR(inSelection), NULL);
+	g_return_val_if_fail(inDirection>XFDASHBOARD_SELECTION_TARGET_NONE, NULL);
+	g_return_val_if_fail(inDirection<=XFDASHBOARD_SELECTION_TARGET_NEXT, NULL);
+	g_return_val_if_fail(inCurrentProviderIter, NULL);
+	g_return_val_if_fail(inNextContainerDirection>XFDASHBOARD_SELECTION_TARGET_NONE, NULL);
+	g_return_val_if_fail(inNextContainerDirection<=XFDASHBOARD_SELECTION_TARGET_NEXT, NULL);
+
+	/* Ask current provider to find selection for requested direction */
+	newSelection=xfdashboard_search_result_container_find_selection(inContainer,
+																	inSelection,
+																	inDirection,
+																	XFDASHBOARD_VIEW(self),
+																	FALSE);
+
+	/* If current provider does not return a matching selection for requested,
+	 * iterate backwards through providers beginning at current provider and
+	 * return the last actor of first provider having an existing container
+	 * while iterating.
+	 */
+	if(!newSelection)
+	{
+		for(iter=g_list_previous(inCurrentProviderIter); iter && !newSelection; iter=g_list_previous(iter))
+		{
+			providerData=(XfdashboardSearchViewProviderData*)iter->data;
+
+			if(providerData &&
+				providerData->container)
+			{
+				newSelection=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(providerData->container),
+																				NULL,
+																				inNextContainerDirection,
+																				XFDASHBOARD_VIEW(self),
+																				FALSE);
+			}
+		}
+	}
+
+	/* If we still have no new selection found, do the same as above but
+	 * iterate from end of list of providers backwards to current provider.
+	 */
+	if(!newSelection)
+	{
+		for(iter=g_list_last(inCurrentProviderIter); iter && iter!=inCurrentProviderIter && !newSelection; iter=g_list_previous(iter))
+		{
+			providerData=(XfdashboardSearchViewProviderData*)iter->data;
+
+			if(providerData &&
+				providerData->container)
+			{
+				newSelection=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(providerData->container),
+																				NULL,
+																				inNextContainerDirection,
+																				XFDASHBOARD_VIEW(self),
+																				FALSE);
+			}
+		}
+	}
+
+	/* If we still have no selection the last resort is to find a selection
+	 * at current provider but this time allow wrapping.
+	 */
+	if(!newSelection)
+	{
+		newSelection=xfdashboard_search_result_container_find_selection(inContainer,
+																		inSelection,
+																		inDirection,
+																		XFDASHBOARD_VIEW(self),
+																		TRUE);
+	}
+
+	/* Return selection found which may be NULL */
+	return(newSelection);
+}
+
+static ClutterActor* _xfdashboard_search_view_focusable_find_selection_internal_forwards(XfdashboardSearchView *self,
+																							XfdashboardSearchResultContainer *inContainer,
+																							ClutterActor *inSelection,
+																							XfdashboardSelectionTarget inDirection,
+																							GList *inCurrentProviderIter,
+																							XfdashboardSelectionTarget inNextContainerDirection)
+{
+	ClutterActor							*newSelection;
+	GList									*iter;
+	XfdashboardSearchViewProviderData		*providerData;
+
+	g_return_val_if_fail(XFDASHBOARD_IS_SEARCH_VIEW(self), NULL);
+	g_return_val_if_fail(XFDASHBOARD_IS_SEARCH_RESULT_CONTAINER(inContainer), NULL);
+	g_return_val_if_fail(CLUTTER_IS_ACTOR(inSelection), NULL);
+	g_return_val_if_fail(inDirection>XFDASHBOARD_SELECTION_TARGET_NONE, NULL);
+	g_return_val_if_fail(inDirection<=XFDASHBOARD_SELECTION_TARGET_NEXT, NULL);
+	g_return_val_if_fail(inCurrentProviderIter, NULL);
+	g_return_val_if_fail(inNextContainerDirection>XFDASHBOARD_SELECTION_TARGET_NONE, NULL);
+	g_return_val_if_fail(inNextContainerDirection<=XFDASHBOARD_SELECTION_TARGET_NEXT, NULL);
+
+	/* Ask current provider to find selection for requested direction */
+	newSelection=xfdashboard_search_result_container_find_selection(inContainer,
+																	inSelection,
+																	inDirection,
+																	XFDASHBOARD_VIEW(self),
+																	FALSE);
+
+	/* If current provider does not return a matching selection for requested,
+	 * iterate forwards through providers beginning at current provider and
+	 * return the last actor of first provider having an existing container
+	 * while iterating.
+	 */
+	if(!newSelection)
+	{
+		for(iter=g_list_next(inCurrentProviderIter); iter && !newSelection; iter=g_list_next(iter))
+		{
+			providerData=(XfdashboardSearchViewProviderData*)iter->data;
+
+			if(providerData &&
+				providerData->container)
+			{
+				newSelection=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(providerData->container),
+																				NULL,
+																				inNextContainerDirection,
+																				XFDASHBOARD_VIEW(self),
+																				FALSE);
+			}
+		}
+	}
+
+	/* If we still have no new selection found, do the same as above but
+	 * iterate from start of list of providers forwards to current provider.
+	 */
+	if(!newSelection)
+	{
+		for(iter=g_list_first(inCurrentProviderIter); iter && iter!=inCurrentProviderIter && !newSelection; iter=g_list_next(iter))
+		{
+			providerData=(XfdashboardSearchViewProviderData*)iter->data;
+
+			if(providerData &&
+				providerData->container)
+			{
+				newSelection=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(providerData->container),
+																				NULL,
+																				inNextContainerDirection,
+																				XFDASHBOARD_VIEW(self),
+																				FALSE);
+			}
+		}
+	}
+
+	/* If we still have no selection the last resort is to find a selection
+	 * at current provider but this time allow wrapping.
+	 */
+	if(!newSelection)
+	{
+		newSelection=xfdashboard_search_result_container_find_selection(inContainer,
+																		inSelection,
+																		inDirection,
+																		XFDASHBOARD_VIEW(self),
+																		TRUE);
+	}
+
+	/* Return selection found which may be NULL */
+	return(newSelection);
+}
+
 static ClutterActor* _xfdashboard_search_view_focusable_find_selection(XfdashboardFocusable *inFocusable,
 																				ClutterActor *inSelection,
 																				XfdashboardSelectionTarget inDirection)
@@ -1308,7 +1237,8 @@ static ClutterActor* _xfdashboard_search_view_focusable_find_selection(Xfdashboa
 				newSelection=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(providerData->container),
 																				NULL,
 																				XFDASHBOARD_SELECTION_TARGET_FIRST,
-																				XFDASHBOARD_VIEW(self));
+																				XFDASHBOARD_VIEW(self),
+																				FALSE);
 				if(newSelection) newSelectionProvider=providerData;
 			}
 		}
@@ -1338,7 +1268,8 @@ static ClutterActor* _xfdashboard_search_view_focusable_find_selection(Xfdashboa
 				newSelection=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(providerData->container),
 																				inSelection,
 																				XFDASHBOARD_SELECTION_TARGET_FIRST,
-																				XFDASHBOARD_VIEW(self));
+																				XFDASHBOARD_VIEW(self),
+																				FALSE);
 				if(newSelection) newSelectionProvider=providerData;
 			}
 		}
@@ -1368,7 +1299,8 @@ static ClutterActor* _xfdashboard_search_view_focusable_find_selection(Xfdashboa
 				newSelection=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(providerData->container),
 																				inSelection,
 																				XFDASHBOARD_SELECTION_TARGET_LAST,
-																				XFDASHBOARD_VIEW(self));
+																				XFDASHBOARD_VIEW(self),
+																				FALSE);
 				if(newSelection) newSelectionProvider=providerData;
 			}
 		}
@@ -1413,117 +1345,33 @@ static ClutterActor* _xfdashboard_search_view_focusable_find_selection(Xfdashboa
 		case XFDASHBOARD_SELECTION_TARGET_UP:
 		case XFDASHBOARD_SELECTION_TARGET_PAGE_LEFT:
 		case XFDASHBOARD_SELECTION_TARGET_PAGE_UP:
-			/* Ask current provider to find selection for requested direction */
-			newSelection=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(priv->selectionProvider->container),
-																			inSelection,
-																			inDirection,
-																			XFDASHBOARD_VIEW(self));
-
-			/* If current provider does not return a matching selection for requested,
-			 * iterate backwards through providers beginning at current provider and
-			 * return the last actor of first provider having an existing container
-			 * while iterating.
-			 */
-			if(!newSelection)
-			{
-				for(iter=g_list_previous(currentProviderIter); iter && !newSelection; iter=g_list_previous(iter))
-				{
-					providerData=(XfdashboardSearchViewProviderData*)iter->data;
-
-					if(providerData &&
-						providerData->container)
-					{
-						newSelection=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(providerData->container),
-																						NULL,
-																						XFDASHBOARD_SELECTION_TARGET_LAST,
-																						XFDASHBOARD_VIEW(self));
-					}
-				}
-			}
+			newSelection=_xfdashboard_search_view_focusable_find_selection_internal_backwards(self,
+																								XFDASHBOARD_SEARCH_RESULT_CONTAINER(priv->selectionProvider->container),
+																								inSelection,
+																								inDirection,
+																								currentProviderIter,
+																								XFDASHBOARD_SELECTION_TARGET_LAST);
 			break;
 
 		case XFDASHBOARD_SELECTION_TARGET_RIGHT:
 		case XFDASHBOARD_SELECTION_TARGET_DOWN:
 		case XFDASHBOARD_SELECTION_TARGET_PAGE_RIGHT:
 		case XFDASHBOARD_SELECTION_TARGET_PAGE_DOWN:
-			/* Ask current provider to find selection for requested direction */
-			newSelection=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(priv->selectionProvider->container),
-																			inSelection,
-																			inDirection,
-																			XFDASHBOARD_VIEW(self));
-
-			/* If current provider does not return a matching selection for requested,
-			 * iterate forwards through providers beginning at current provider and
-			 * return the first actor of first provider having an existing container
-			 * while iterating.
-			 */
-			if(!newSelection)
-			{
-				for(iter=g_list_next(currentProviderIter); iter && !newSelection; iter=g_list_next(iter))
-				{
-					providerData=(XfdashboardSearchViewProviderData*)iter->data;
-
-					if(providerData &&
-						providerData->container)
-					{
-						newSelection=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(providerData->container),
-																						NULL,
-																						XFDASHBOARD_SELECTION_TARGET_FIRST,
-																						XFDASHBOARD_VIEW(self));
-					}
-				}
-			}
+			newSelection=_xfdashboard_search_view_focusable_find_selection_internal_forwards(self,
+																								XFDASHBOARD_SEARCH_RESULT_CONTAINER(priv->selectionProvider->container),
+																								inSelection,
+																								inDirection,
+																								currentProviderIter,
+																								XFDASHBOARD_SELECTION_TARGET_FIRST);
 			break;
 
 		case XFDASHBOARD_SELECTION_TARGET_NEXT:
-			/* Ask current provider to find selection for requested direction */
-			newSelection=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(priv->selectionProvider->container),
-																			inSelection,
-																			inDirection,
-																			XFDASHBOARD_VIEW(self));
-
-			/* If current provider does not return a matching selection for requested,
-			 * iterate forwards through providers beginning at current provider and
-			 * return the first actor of first provider having an existing container
-			 * while iterating.
-			 */
-			if(!newSelection)
-			{
-				for(iter=g_list_next(currentProviderIter); iter && !newSelection; iter=g_list_next(iter))
-				{
-					providerData=(XfdashboardSearchViewProviderData*)iter->data;
-
-					if(providerData &&
-						providerData->container)
-					{
-						newSelection=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(providerData->container),
-																						NULL,
-																						XFDASHBOARD_SELECTION_TARGET_FIRST,
-																						XFDASHBOARD_VIEW(self));
-					}
-				}
-			}
-
-			/* If still no matching selection was found then iterate backwards through
-			 * providers beginning at current provider and return the last actor of first
-			 * provider having an existing container while iterating.
-			 */
-			if(!newSelection)
-			{
-				for(iter=g_list_previous(currentProviderIter); iter && !newSelection; iter=g_list_previous(iter))
-				{
-					providerData=(XfdashboardSearchViewProviderData*)iter->data;
-
-					if(providerData &&
-						providerData->container)
-					{
-						newSelection=xfdashboard_search_result_container_find_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(providerData->container),
-																						NULL,
-																						XFDASHBOARD_SELECTION_TARGET_LAST,
-																						XFDASHBOARD_VIEW(self));
-					}
-				}
-			}
+			newSelection=_xfdashboard_search_view_focusable_find_selection_internal_forwards(self,
+																								XFDASHBOARD_SEARCH_RESULT_CONTAINER(priv->selectionProvider->container),
+																								inSelection,
+																								inDirection,
+																								currentProviderIter,
+																								XFDASHBOARD_SELECTION_TARGET_FIRST);
 			break;
 
 		case XFDASHBOARD_SELECTION_TARGET_FIRST:
@@ -1583,7 +1431,8 @@ static gboolean _xfdashboard_search_view_focusable_activate_selection(Xfdashboar
 	}
 
 	/* Activate selection */
-	_xfdashboard_search_view_on_provider_item_actor_clicked(NULL, inSelection, providerData);
+	xfdashboard_search_result_container_activate_selection(XFDASHBOARD_SEARCH_RESULT_CONTAINER(providerData->container),
+															inSelection);
 
 	/* Release allocated resources */
 	_xfdashboard_search_view_provider_data_unref(providerData);
@@ -1798,9 +1647,6 @@ void xfdashboard_search_view_reset_search(XfdashboardSearchView *self)
 		/* Get data for provider to reset */
 		providerData=((XfdashboardSearchViewProviderData*)(iter->data));
 
-		/* Clean provider's container and move focus if necessary */
-		_xfdashboard_search_view_clean_provider_container(providerData);
-
 		/* Destroy container */
 		if(providerData->container)
 		{
